﻿#pragma once
#include "Planets.h"

//элементы радиус-вектора в экаваториальной системе координат
struct equator_elements_orbit {
	double r; // величина радиус-вектора
	double v; // линенйая скорость
	double θ; // угол между направлением линейной скорости и трансверсалью орбиты
	double i; // наклонение орбиты
	double Ω; //а долгота восходящего узла орбиты
	double u; // аргумент широты
};
//элементы радиус-вектора в меридиальной системе координат
struct meridian_elements_orbit{
	double r; // величина радиус-вектора
	double v; // линенйая скорость
	double θ; // угол между направлением линейной скорости и трансверсалью орбиты
	double i; // наклонение орбиты
	double Ω; // долгота восходящего узла орбиты
	double u; // аргумент широты
};
//Кеплеровские элементы орбиты
struct Kepler_elements_orbit {
	double a; //величина большой полуоси орбиты
	double e; //эксцентриситет орбиты
	double i; //угол наклонения орбиты в градусах
	double Ω; //долгота восходящего узла в градусах
	double ω; //аргумент перигея в градусах
	double M; //величина средней аномалии градусах
	double E; //величина эксцентрической аномалии
	double N; //истинная аномалия
	double u; //аргумент широты
	double convertGradToRad(double grad);
	double convertRadToGrad(double rad);
};

class Satellite {
private:
	static int satellite_number;
	const int m_Id;
	const double Radius_Earth = 6378.13630; // средний радиус земли
	const double compression_earth = 1 / 298.257; //сжатие Земли
	static const std::vector<double> coef_time_step;
	double m_start_time; //момент запуска спутника
	double last_moment_integration; // ммомент времени после последнего интегрирования
	
	satellite_parameters m_param{1,1,1.5};
	double m_r{ 0 }; // радиус-вектор полета спутника в данный момент времени
	double m_v{ 0 }; // вектор скорости полета спутника в данный момент времени
	double m_h{ 0 }; // высота полета спуттника в данный момент времени
	double m_density{0}; //плотность атмосферы на данной высоте

	Matrix m_DCS{ 3, 1 }; // геоцентрические небесные координаты спутника
	Matrix m_DVS{ 3, 1 }; // геоцентирические небесные компоненты скорости
	Matrix m_F{ 3, 1 }; // компоненты ускорения вызванного атмосферой Земли
	
	equator_elements_orbit m_eeo{}; //экватариальный вектор состояния
	meridian_elements_orbit m_meo{}; //меридиальный вектор состояния
	Kepler_elements_orbit m_Keo{}; // Кеплеровские элементы орбиты

	std::vector<Matrix> m_α{}; //массив хранящий коэффициенты полинома

	Moon* m_moon{nullptr};
	Sun* m_sun{ nullptr };
	Earth* m_earth{ nullptr };
	
	// вычисленеие радиус-вектора полета спутника
	int calculate_r(); 
	// вычисленеи плотности амосферы
	int calculate_density(const Matrix& DCS);
	//вычисление вектора скорости спутника
	int calculate_V(const Matrix& DVS);
	// вычисление ускорения обусловленного сопротивлением атмосферы
	Matrix& calculate_F();
	// вычисление ускорения обусловленного сопротивлением атмосферы
	Matrix& recalculate_F(const Matrix& i_DCS,const Matrix& i_DVS);
	// вычисление всех ускорений на заданный момент времени
	Matrix step_calculate_F(const Matrix& i_DCS, const Matrix& i_DVS, const double step_time);
	
	//преобразовние экватариального вектора состояния в декартову ситему
	int convertStateVectorToDCe();
	//преобразовние мериадильного вектора состояния в декартову ситему
	int convertStateVectorToDCm();
	//преобразование декартовой системы в эватариальный вектор состояния
	int convertDCeTOStateVector();
	//преобразования декартовой системы в меридиальный вектор состояния
	int convertDCmTOSteteVector();
	//преобразование Кеплеровских элементов орбиты в декартовую систму координат
	int convertKeplertoDC();
	//Инициализация массива коэффициентов полинома
	int initialization_A();
public:
	
	Satellite(double time_MJD, satellite_parameters& i_param, Matrix& i_DCS, Matrix& i_DVS, Moon* ptr_moon, Sun* ptr_sun, Earth* ptr_earth);
	Satellite(double time_MJD, satellite_parameters& i_param, Kepler_elements_orbit& i_Keo, Moon* ptr_moon, Sun* ptr_sun, Earth* ptr_earth);
	//интегрирование на один шаг времени, в качестве параметра конечное время 
	int integration_step(double i_time);
	const Matrix& getDCS() { return m_DCS; };
	const Matrix& getDVS() { return m_DVS; };
	const double getRadius() { return m_r; };
	const double getOmega() { return m_eeo.Ω; };
	const double getU() { return m_eeo.u; };
	int getId() { return m_Id; };
	~Satellite() {};
};
